# Copyright 2004 Daniel Henninger <jadestorm@nc.rr.com>
# Licensed for distribution under the GPL version 2, check COPYING for details

from twisted.internet import protocol, reactor, defer
from tlib import oscar
from tlib.domish import Element
from tlib import socks5, sockserror
from twisted.python import log
import config
import utils
import debug
import sys, warnings, pprint
import stats
import lang
import re



#############################################################################
# BOSConnection
#############################################################################
class B(oscar.BOSConnection):
	def __init__(self,username,cookie,aimcon):
		self.chats = list()
		self.aimcon = aimcon
		self.authorizationRequests = [] # buddies that need authorization
		self.aimcon.bos = self
		self.session = aimcon.session  # convenience
		self.capabilities = [oscar.CAP_CHAT]
		if (config.crossChat):
			self.capabilities.append(oscar.CAP_CROSS_CHAT)
		if (config.socksProxyServer and config.socksProxyPort):
			self.socksProxyServer = config.socksProxyServer
			self.socksProxyPort = config.socksProxyPort
		oscar.BOSConnection.__init__(self,username,cookie)

	def initDone(self):
		self.requestSelfInfo().addCallback(self.gotSelfInfo)
		self.requestSSI().addCallback(self.gotBuddyList)
		debug.log("B: initDone %s for %s" % (self.username,self.session.jabberID))

        def connectionLost(self, reason):
                oscar.BOSConnection.connectionLost(self,reason)
                self.session.removeMe();

	def buddyAdded(self, scrnname):
		from glue import aim2jid
		for g in self.ssigroups:
			for u in g.users:
				if u.name == scrnname:
					if u.authorized:
						self.session.sendPresence(to=self.session.jabberID, fro=aim2jid(scrnname), show=None, ptype="subscribed")
						return

	def gotAuthorizationRespons(self, scrnname, success):
		from glue import aim2jid
		debug.log("B: Authorization Respons: %s, %s"%(scrnname, success))
		if success:
			for g in self.ssigroups:
				for u in g.users:
					if u.name == scrnname:
						u.authorized = True
						u.authorizationRequestSent = False
						self.session.sendPresence(to=self.session.jabberID, fro=aim2jid(scrnname), show=None, ptype="subscribed")
						return
		else:
			for g in self.ssigroups:
				for u in g.users:
					if u.name == scrnname:
						u.authorizationRequestSent = False
			self.session.sendPresence(to=self.session.jabberID, fro=aim2jid(scrnname), show=None, status=None, ptype="unsubscribed")

	def gotAuthorizationRequest(self, scrnname):
		from glue import aim2jid
		debug.log("B: Authorization Request: %s"%scrnname)
		if not scrnname in self.authorizationRequests:
			self.authorizationRequests.append(scrnname)
			self.session.sendPresence(to=self.session.jabberID, fro=aim2jid(scrnname), ptype="subscribe")

	def youWereAdded(self, scrnname):
		from glue import aim2jid
		debug.log("B: %s added you to her/his contact list"%scrnname)
		self.session.sendPresence(to=self.session.jabberID, fro=aim2jid(scrnname), ptype="subscribe")

	def updateBuddy(self, user):
		from glue import aim2jid

		debug.log("B: updateBuddy %s" % (user))
		buddyjid = aim2jid(user.name)
		ptype = None
		show = None
                status = user.status
                status = re.sub("<[^>]*>","",status); # Removes any HTML tags
                if status == "Away" or status=="I am currently away from the computer." or status=="I am away from my computer right now.":
                        status = ""
                if(user.idleTime):
                        if(user.idleTime>60*24):
                                idle_time = "Idle %d days"%(user.idleTime/(60*24))
                        elif (user.idleTime>60):
                                idle_time = "Idle %d hours"%(user.idleTime/(60))
                        else:
                                idle_time = "Idle %d minutes"%(user.idleTime)
                        if(status):
                                status="%s - %s"%(idle_time,status)
                        else:
                                status=idle_time

		if (user.flags.count("away")):
			self.getAway(user.name).addCallback(self.sendAwayPresence, user)
		else:
			self.session.sendPresence(to=self.session.jabberID, fro=buddyjid, show=show, status=status, ptype=ptype)
			self.aimcon.contacts.updateSSIContact(user.name, presence=ptype, show=show, status=status)

	def offlineBuddy(self, user):
		from glue import aim2jid

		debug.log("B: offlineBuddy %s" % (user.name))
		buddyjid = aim2jid(user.name)
		show = None
		status = None
		ptype = "unavailable"
		self.session.sendPresence(to=self.session.jabberID, fro=buddyjid, show=show, status=status, ptype=ptype)

	def receiveMessage(self, user, multiparts, flags):
		from glue import aim2jid

		debug.log("B: receiveMessage %s %s %s %s %s" % (self.session.jabberID, self.name, user.name, multiparts, flags))
		sourcejid = aim2jid(user.name)
		text = oscar.dehtml(multiparts[0][0])
		text = text.decode("utf-8", "replace")
		text = text.strip()
		self.session.sendMessage(to=self.session.jabberID, fro=sourcejid, body=text, mtype="chat")
		stats.incmessages += 1
		stats.sessionUpdate(self.session.jabberID, "incmessages", 1)        

	def receiveWarning(self, newLevel, user):
		debug.log("B: receiveWarning [%s] from %s" % (newLevel,hasattr(user,'name') and user.name or None))

	def receiveTypingNotify(self, type, user):
		from glue import aim2jid

		debug.log("B: receiveTypingNotify %s from %s" % (type,hasattr(user,'name') and user.name or None))
		sourcejid = aim2jid(user.name)
		if (type == "begin"):
			self.session.sendTypingNotification(to=self.session.jabberID, fro=sourcejid, typing=True)
		elif (type == "idle"):
			self.session.sendTypingNotification(to=self.session.jabberID, fro=sourcejid, typing=False)
		elif (type == "finish"):
			self.session.sendTypingNotification(to=self.session.jabberID, fro=sourcejid, typing=False)

	def receiveChatInvite(self, user, message, exchange, fullName, instance, shortName, inviteTime):
		from glue import LegacyGroupchat, aim2jid

		debug.log("B: receiveChatInvite from %s for room %s with message: %s" % (user.name,shortName,message))
		groupchat = LegacyGroupchat(session=self.session, resource=None, ID="#"+shortName.replace(' ','_')+"%"+str(exchange), existing=True)
		groupchat.sendUserInvite(aim2jid(user.name))

	def chatReceiveMessage(self, chat, user, message):
		from glue import aim2jidGroup

		debug.log("B: chatReceiveMessage to %s:%d from %s:%s" % (chat.name,chat.exchange,user.name,message))

		if (user.name.lower() == self.username.lower()):
			return

		fro = aim2jidGroup(chat.name, user.name, chat.exchange)
		text = oscar.dehtml(message)
		text = text.decode("utf-8", "replace")
		text = text.strip()
		self.session.sendMessage(to=self.session.jabberID, fro=fro, body=text, mtype="groupchat")
		stats.incmessages += 1
		stats.sessionUpdate(self.session.jabberID, "incmessages", 1)

	def chatMemberJoined(self, chat, member):
		from glue import aim2jidGroup

		debug.log("B: chatMemberJoined %s joined %s" % (member.name,chat.name))
		fro = aim2jidGroup(chat.name, member.name, chat.exchange)
		ptype = None
		show = None
		status = None
		self.session.sendPresence(to=self.session.jabberID, fro=fro, show=show, status=status, ptype=ptype)

	def chatMemberLeft(self, chat, member):
		from glue import aim2jidGroup

		debug.log("B: chatMemberLeft %s left %s (members: %s)" % (member.name,chat.name,map(lambda x:x.name,chat.members)))
		fro = aim2jidGroup(chat.name, member.name, chat.exchange)
		ptype = "unavailable"
		show = None
		status = None
		self.session.sendPresence(to=self.session.jabberID, fro=fro, show=show, status=status, ptype=ptype)

	def receiveSendFileRequest(self, user, file, description, cookie):
		debug.log("B: receiveSendFileRequest")


	# Callbacks
	def sendAwayPresence(self, msg, user):
		from glue import aim2jid
        
		buddyjid = aim2jid(user.name)
		ptype = None
		show = "away"
		status = msg
                #status = oscar.dehtml(msg); # Removes any HTML tags
                status = re.sub("<[^>]*>","",status); # Removes any HTML tags
                if status == "Away" or status=="I am currently away from the computer." or status=="I am away from my computer right now.":
                        status = ""
                if(user.idleTime):
                        if(user.idleTime>60*24):
                                idle_time = "Idle %d days"%(user.idleTime/(60*24))
                        elif (user.idleTime>60):
                                idle_time = "Idle %d hours"%(user.idleTime/(60))
                        else:
                                idle_time = "Idle %d minutes"%(user.idleTime)
                        if(status):
                                status="%s - %s"%(idle_time,status)
                        else:
                                status=idle_time
		self.session.sendPresence(to=self.session.jabberID, fro=buddyjid, show=show, status=status, ptype=ptype)
		self.aimcon.contacts.updateSSIContact(user.name, presence=ptype, show=show, status=status)

	def gotSelfInfo(self, user):
		debug.log("B: gotSelfInfo: %s" % (user.__dict__))
		self.name = user.name

	def gotBuddyList(self, l):
		debug.log("B: gotBuddyList: %s" % (str(l)))
		self.ssigroups = list()
		if (l is not None and l[0] is not None):
			for g in l[0]:
				debug.log("B: gotBuddyList found group %s" % (g.name))
				self.ssigroups.append(g)
				for u in g.users:
					debug.log("B: got user %s from group %s" % (u.name, g.name))
					self.aimcon.contacts.updateSSIContact(u.name, skipsave=True)
		self.aimcon.contacts.saveXDBBuddies()
		self.activateSSI()
		self.setProfile(None)
		self.setIdleTime(0)
		self.clientReady()
		self.session.ready = True
		tmpjid=config.jid
		if(self.session.registeredmunge):
			tmpjid=config.jid+"/registered"
		self.session.sendPresence(to=self.session.jabberID, fro=tmpjid, show=self.aimcon.savedShow, status=self.aimcon.savedFriendly)
		if (self.aimcon.savedShow in ["online", None]):
			self.aimcon.setAway(None)
		else:
			self.aimcon.setAway(self.aimcon.savedFriendly)

	def warnedUser(self, oldLevel, newLevel, username):
		debug.log("B: warnedUser");

	def createdRoom(self, (exchange, fullName, instance)):
		debug.log("B: createdRoom: %s, %s, %s" % (exchange, fullName, instance))
		self.joinChat(exchange, fullName, instance).addCallback(self.chatJoined)

	def chatJoined(self, chat):
		from glue import aim2jidGroup

		debug.log("B: chatJoined room %s (members: %s)" % (chat.name,map(lambda x:x.name,chat.members)))
		if (chat.members is not None):
			for m in chat.members:
				fro = aim2jidGroup(chat.name, m.name, chat.exchange)
				ptype = None
				show = None
				status = None
				self.session.sendPresence(to=self.session.jabberID, fro=fro, show=show, status=status, ptype=ptype)
		self.chats.append(chat)



#############################################################################
# Oscar Authenticator
#############################################################################
class OA(oscar.OscarAuthenticator):
	def __init__(self,username,password,aimcon,deferred=None,icq=0):
		self.aimcon = aimcon
		self.BOSClass = B
		oscar.OscarAuthenticator.__init__(self,username,password,deferred,icq)

	def connectToBOS(self, server, port):
		if config.socksProxyServer:
			c = socks5.ProxyClientCreator(reactor, self.BOSClass, self.username, self.cookie, self.aimcon)
			return c.connectSocks5Proxy(server, int(port), config.socksProxyServer, int(config.socksProxyPort), "OABOS")
		else:
			c = protocol.ClientCreator(reactor, self.BOSClass, self.username, self.cookie, self.aimcon)
			return c.connectTCP(server, int(port))



#############################################################################
# AIMConnection
#############################################################################
class AIMConnection:
	def __init__(self, username, password):
		self.username = username
		self.password = password
		self.reactor = reactor
		self.contacts = AIMContacts(self.session)
		self.deferred = defer.Deferred()
		self.deferred.addErrback(self.errorCallback)
		hostport = (config.aimServer, int(config.aimPort))
		debug.log("AIMConnection: client creation for %s" % (self.session.jabberID))
		if config.socksProxyServer and config.socksProxyPort:
			self.oa = OA
			self.creator = socks5.ProxyClientCreator(self.reactor, self.oa, self.username, self.password, self, deferred=self.deferred, icq=0)
			debug.log("AIMConnection: connect via socks proxy")
			self.creator.connectSocks5Proxy(config.aimServer, int(config.aimPort), config.socksProxyServer, int(config.socksProxyPort), "AIMCONN")
		else:
			self.oa = OA
			self.creator = protocol.ClientCreator(self.reactor, self.oa, self.username, self.password, self, deferred=self.deferred, icq=0)
			debug.log("AIMConnection: connect direct tcp")
			self.creator.connectTCP(*hostport)

		debug.log("AIMConnection: \"%s\" created" % (self.username))
	
	def setAway(self, awayMessage=None):
		debug.log("AIMConnection: setAway %s" % (awayMessage))
		try:
			self.bos.setAway(awayMessage)
		except AttributeError:
			#self.alertUser(lang.get(config.jid).sessionnotactive)
			pass

        def setBack(self, backMessage=None):
                debug.log("AIMConnection: setBack %s" % (backMessage))
                try:
                        self.bos.setBack(backMessage)
                except AttributeError:
                        #self.alertUser(lang.get(config.jid).sessionnotactive)
                        pass

        def setProfile(self, profileMessage=None):
                debug.log("AIMConnection: setProfile %s" % (profileMessage))
                try:
                        self.bos.setProfile(profileMessage)
                except AttributeError:
                        #self.alertUser(lang.get(config.jid).sessionnotactive)
                        pass

	def sendMessage(self, target, message):
		from glue import jid2aim
		try:
			stats.outmessages += 1
			stats.sessionUpdate(self.session.jabberID, "outmessages", 1)        
			scrnname = jid2aim(target)
			debug.log("AIMConnection: sendMessage %s %s" % (scrnname, message))
			if (scrnname[0].isdigit()):
				self.bos.sendMessage(scrnname, message.encode("iso-8859-1", "replace"))
			else:
				htmlized = oscar.html(message.encode("iso-8859-1", "replace"))
				self.bos.sendMessage(scrnname, htmlized)
		except AttributeError:
			self.alertUser(lang.get(config.jid).sessionnotactive)

	def resendBuddies(self, resource):
		from glue import aim2jid
		debug.log("AIMConnection: resendBuddies %s" % (resource))
		try:
			for c in self.contacts.ssicontacts.keys( ):
				debug.log("AIMConnection: resending buddy of %s" % (c))
				jid = aim2jid(c)
				show = self.contacts.ssicontacts[c]['show']
				status = self.contacts.ssicontacts[c]['status']
				ptype = self.contacts.ssicontacts[c]['presence']
				self.session.sendPresence(to=self.session.jabberID, fro=jid, show=show, status=status, ptype=ptype)
		except AttributeError:
			return

	def sendTypingNotify(self, type, dest):
		from tlib.oscar import MTN_FINISH, MTN_IDLE, MTN_BEGIN
		from glue import jid2aim
		try:
			username = jid2aim(dest)
			debug.log("AIMConnection: sendTypingNotify %s to %s" % (type,username))
			if (type == "begin"):
				self.bos.sendTypingNotification(username, MTN_BEGIN)
			elif (type == "idle"):
				self.bos.sendTypingNotification(username, MTN_IDLE)
			elif (type == "finish"):
				self.bos.sendTypingNotification(username, MTN_FINISH)
		except AttributeError:
			self.alertUser(lang.get(config.jid).sessionnotactive)

	def createChat(self, chatroom, exchange):
		debug.log("AIMConnection: createChat %s %d" % (chatroom, exchange))
		try:
			self.bos.createChat(chatroom, exchange).addCallback(self.bos.createdRoom)
		except AttributeError:
			self.alertUser(lang.get(config.jid).sessionnotactive)

	def leaveChat(self, chatroom):
		debug.log("AIMConnection: leaveChat %s" % (chatroom))
		try:
			for c in self.bos.chats:
				if (c.name == chatroom):
					c.leaveChat()
					self.bos.chats.remove(c)
					break
		except AttributeError:
			self.alertUser(lang.get(config.jid).sessionnotactive)

	def sendChat(self, chatroom, message):
		debug.log("AIMConnection: sendChat %s %s" % (chatroom, message))
		try:
			for c in self.bos.chats:
				if (c.name.lower() == chatroom.lower()):
					c.sendMessage(message)
					break
		except AttributeError:
			self.alertUser(lang.get(config.jid).sessionnotactive)

	def sendInvite(self, chatroom, contact):
		debug.log("AIMConnection: sendInvite %s %s" % (chatroom, contact))
		try:
			for c in self.bos.chats:
				if (c.name.lower() == chatroom.lower()):
					self.bos.sendInvite(contact, c)
		except AttributeError:
			self.alertUser(lang.get(config.jid).sessionnotactive)

	def getvCard(self, vcard, user):
		debug.log("AIMConnection: getvCard %s" % (user))
		try:
			d = defer.Deferred()
			self.bos.getProfile(user).addCallback(self.gotvCard, user, vcard, d).addErrback(self.gotnovCard, user, vcard, d)
			return d
		except AttributeError:
			self.alertUser(lang.get(config.jid).sessionnotactive)

	def gotvCard(self, profile, user, vcard, d):
		from glue import aim2jid
		debug.log("AIMConnection: gotvCard: %s" % (profile))

		profilestart = profile.find("<HTML>")
		cutprofile = oscar.dehtml(profile[profilestart:])
		nickname = vcard.addElement("NICKNAME")
		nickname.addContent(user)
		jabberid = vcard.addElement("JABBERID")
		jabberid.addContent(aim2jid(user))
		desc = vcard.addElement("DESC")
		desc.addContent(cutprofile)

		d.callback(vcard)

	def gotnovCard(self, profile, user, vcard, d):
		from glue import aim2jid
		debug.log("AIMConnection: novCard: %s" % (profile))

		nickname = vcard.addElement("NICKNAME")
		nickname.addContent(user)
		jabberid = vcard.addElement("JABBERID")
		jabberid.addContent(aim2jid(user))
		desc = vcard.addElement("DESC")
		desc.addContent("User is not online.")

		d.callback(vcard)

	def removeMe(self):
		from glue import aim2jid
		debug.log("AIMConnection: removeMe")
		try:
			self.bos.stopKeepAlive()
			self.bos.disconnect()

			for c in self.contacts.ssicontacts.keys( ):
				debug.log("AIMConnection: sending offline for %s" % (c))
				jid = aim2jid(c)
				show = None
				status = None
				ptype = "unavailable"
				self.session.sendPresence(to=self.session.jabberID, fro=jid, show=show, status=status, ptype=ptype)
		except AttributeError:
			return

	def jabberSubscriptionReceived(self, to, subtype):
		debug.log("AIMConnection: Session \"%s\" - jabberSubscriptionReceived(\"%s\", \"%s\")" % (self.session.jabberID, to, subtype))

		def aim2uhandle(aimid):
			retstr = aimid.replace(' ','')
			return retstr.lower()

		def updatePresence(ptype): # Convenience
			self.session.sendPresence(to=self.session.jabberID, fro=to, ptype=ptype)

		if(to.find('@') > 0): # For contacts
			from glue import jid2aim

			userHandle = jid2aim(to)

			if(subtype == "subscribe"):
				# User wants to subscribe to contact's presence
				debug.log("AIMConnection: Subscribe request received.")
				def cb(arg=None):
					updatePresence("subscribed")

				try:
					for g in self.bos.ssigroups:
						for u in g.users:
							aimHandle = aim2uhandle(u.name)
							if aimHandle == userHandle:
								if (not u.authorizationRequestSent) and (not u.authorized):
									self.bos.sendAuthorizationRequest(userHandle, "Please authorize me!")
									u.authorizationRequestSent = True
									return
								else:
									cb()
									return

					savethisgroup = None
					groupName = "PyAIM-t Buddies"
					for g in self.bos.ssigroups:
						if (g.name == groupName):
							debug.log("Located group %s" % (g.name))
							savethisgroup = g

					if (savethisgroup is None):
						debug.log("Adding new group")
						newGroupID = self.generateGroupID()
						newGroup = oscar.SSIGroup(groupName, newGroupID, 0)
						self.bos.startModifySSI()
						self.bos.addItemSSI(newGroup)
						self.bos.endModifySSI()
						savethisgroup = newGroup
						self.bos.ssigroups.append(newGroup)

					group = self.findGroupByName(groupName)
					newUserID = self.generateBuddyID(group.groupID)
					newUser = oscar.SSIBuddy(userHandle, group.groupID, newUserID)
					savethisgroup.addUser(newUserID, newUser)


					debug.log("Adding item to SSI")
					self.bos.startModifySSI()
					self.bos.addItemSSI(newUser)
					self.bos.modifyItemSSI(savethisgroup)
					self.bos.endModifySSI()

					self.contacts.updateSSIContact(userHandle)
					updatePresence("subscribe")
				except AttributeError:
					self.alertUser(lang.get(config.jid).sessionnotactive)

			elif(subtype == "subscribed"):
				# The user has granted this contact subscription
				debug.log("AIMConnection: Subscribed request received.")
				try:
					if userHandle in self.bos.authorizationRequests:
						self.bos.sendAuthorizationRespons(userHandle, True, "OK")
						self.bos.authorizationRequests.remove(userHandle)
				except AttributeError:
					self.alertUser(lang.get(config.jid).sessionnotactive)

			elif(subtype == "unsubscribe" or subtype == "unsubscribed"):
				# User wants to unsubscribe to this contact's presence. (User is removing the contact from their list)
				debug.log("AIMConnection: Unsubscribe(d) request received.")
				if userHandle in self.bos.authorizationRequests:
					self.bos.sendAuthorizationRespons(userHandle, False, "")
					self.bos.authorizationRequests.remove(userHandle)

				def cb(arg=None):
					updatePresence("unsubscribed")

				try:
	                                savethisuser = None
					for g in self.bos.ssigroups:
						for u in g.users:
							aimHandle = aim2uhandle(u.name)
							debug.log("Comparing %s and %s" % (aimHandle, userHandle))
							if (aimHandle == userHandle):
								debug.log("Located user %s" % (u.name))
								savethisuser = u

					if (savethisuser is None):
						debug.log("Did not find user")
						return

					self.bos.startModifySSI()
					debug.log("AIMConnection: Removing %s (u:%d g:%d) from group %s"%(savethisuser.name, savethisuser.buddyID, savethisuser.groupID, savethisuser.group.name))
					de = self.bos.delItemSSI(savethisuser)
					de.addCallback(self.SSIItemDeleted, savethisuser)
					de.addCallback(cb)
					self.bos.endModifySSI()
				except AttributeError:
					self.alertUser(lang.get(config.jid).sessionnotactive)

                else: # The user wants to change subscription to the transport
			try:
	                        if(subtype == "subscribe"):
					updatePresence("subscribed")

				elif(subtype == "subscribed"):
					return # Nothing to do

				elif(subtype == "unsubscribe" or subtype == "unsubscribed"):
					# They want to unregister. Ok, we can do that
					jid = self.session.jabberID
					debug.log("Subscriptions: Session \"%s\" is about to be unregistered" % (jid))
					self.session.pytrans.registermanager.removeRegInfo(jid)
					debug.log("Subscriptions: Session \"%s\" has been unregistered" % (jid))
			except AttributeError:
				self.alertUser(lang.get(config.jid).sessionnotactive)

	def SSIItemDeleted(self, x, user):
		c = 0
		for g in self.bos.ssigroups:
			c += 1
			for u in g.users:
				if u.name == user.name:
					g.users.remove(u)
					del g.usersToID[u]

	# Callbacks
	def errorCallback(self, result):
		debug.log("AIMConnection: errorCallback %s" % (result.getErrorMessage()))
		errmsg = result.getErrorMessage()
		errmsgs = errmsg.split("'")
		message = "Authentication Error!" 
		if (errmsgs[1]):
			message = message+"\n"+errmsgs[1]
		if (errmsgs[3]):
			message = message+"\n"+errmsgs[3]
		self.alertUser(message)

		self.session.removeMe()

	def findGroupByID(self, groupID):
		for x in self.bos.ssigroups:
			if x.groupID == groupID:
				return x

	def findGroupByName(self, groupName):
		for x in self.bos.ssigroups:
			if x.name == groupName:
				return x

	def generateGroupID(self):
		pGroupID = len(self.bos.ssigroups)
		while True:
			pGroupID += 1
			found = False
			for x in self.bos.ssigroups:
				if pGroupID == x.groupID:
					found = True
					break
			if not found: break
		return pGroupID

	def generateBuddyID(self, groupID):
		group = self.findGroupByID(groupID)
		pBuddyID = len(group.users)
		while True: # If all integers are taken we got a bigger problems
			pBuddyID += 1
			found = False
			for x in group.users:
				if pBuddyID == x.buddyID:
					found = True
					break
			if not found: break
		return pBuddyID

	def alertUser(self, message):
		tmpjid = config.jid
		if (self.session.registeredmunge):
			tmpjid = tmpjid + "/registered"
		self.session.sendMessage(to=self.session.jabberID, fro=tmpjid, body=message, mtype="error")


#############################################################################
# AIMContacts
#############################################################################
class AIMContacts:
	def __init__(self, session):
		self.session = session
		self.ssicontacts = { }
		self.xdbcontacts = self.getXDBBuddies()
		self.xdbchanged = False

	def updateSSIContact(self, contact, presence="unavailable", show=None, status=None, skipsave=False):
		debug.log("AIMContacts: updating contact %s" % (contact.lower()))
		self.ssicontacts[contact.lower()] = {
			'presence': presence,
			'show': show,
			'status': status
		}

		if (not self.xdbcontacts.count(contact.lower())):
			from glue import aim2jid
			self.session.sendRosterImport(aim2jid(contact), "subscribe", "both", contact)
			self.xdbcontacts.append(contact.lower())
			self.xdbchanged = True
			if (not skipsave):
				self.saveXDBBuddies()

	def getXDBBuddies(self):
		debug.log("AIMContacts: getXDBBuddies %s %s" % (config.jid, self.session.jabberID))
		bl = list()
		result = self.session.pytrans.xdb.request(self.session.jabberID, "aimtrans:roster")
		if (result == None):
			debug.log("AIMContacts: getXDBBuddies unable to get list, or empty")
			return bl

		for child in result.elements():
			try:
				if(child.name == "buddies"):
					for item in child.elements():
						try:
							if(item.name == "item"):
								bl.append(item.getAttribute("name"))
						except AttributeError:
							continue
			except AttributeError:
				continue
		return bl

	def saveXDBBuddies(self):
		debug.log("AIMContacts: setXDBBuddies %s %s" % (config.jid, self.session.jabberID))
		if (not self.xdbchanged):
			debug.log("AIMContacts: nothing has changed, no need to save")
			return
		newXDB = Element((None, "aimtrans"))
		newXDB.attributes["xmlns"] = "aimtrans:roster"
		buddies = newXDB.addElement("buddies")

		for c in self.xdbcontacts:
			try:
				item = buddies.addElement("item")
				item.attributes["name"] = c

			except:
				pass

		self.session.pytrans.xdb.set(self.session.jabberID, "aimtrans:roster", newXDB)
		self.xdbchanged = False
		debug.log("AIMContacts: contacts saved")
