# Copyright 2004 Daniel Henninger <jadestorm@nc.rr.com>
# Licensed for distribution under the GPL version 2, check COPYING for details

from twisted.internet import task
from tlib import oscar
from tlib.domish import Element
import groupchat
import aimt
import config
import debug
import utils

# The name of the transport
name = "AIM Transport"

# The transport's version
version = "0.6"

# This should be set to the identity of the gateway
id = "aim"

# This should be set to the identity of the conference gateway, blank if none
confid = "text"

# Set to True if spool directory entries need to be mangled @ -> %
mangle = True

# This should be set to the name space roster entries are in in the spool
namespace = "aimtrans:data"

# This function should return an xml element as it should exist in the spool
def formRegEntry(username, password):
	reginfo = Element((None, "aimtrans"))

	logoninfo = reginfo.addElement("logon")
	logoninfo.attributes["id"] = username
	logoninfo.attributes["pass"] = password

	return reginfo

# This function should, given a spool xml entry, pull the username and password
# out of it and return them
def getAttributes(base):
	username = ""
	password = ""

	for child in base.elements():
		try:
			if(child.name == "logon"):
				username = child.getAttribute("id")
				password = child.getAttribute("pass")
		except AttributeError:
			continue

	return username, password

# This function should return true if the JID is a group JID, false otherwise
def isGroupJID(jid):
	if (jid[0] == "#" or jid[0] == "%"):
		return True
	else:
		return False

# This function translates an AIM screen name to a JID
def aim2jid(aimid):
	if (aimid):
		retstr = aimid.replace(' ', '')
		return retstr.replace('@', '%') + "@" + config.jid
	else:
		return config.jid

# This function translates a JID to an AIM screen name
def jid2aim(jid):
	return unicode(jid[:jid.find('@')].replace('%','@'))

# This function translates an AIM chat room to a groupchat JID
def aim2jidGroup(chatid, userid=None, exchange=None):
	retstr = "#" + chatid.replace(' ', '_')
	retstr = retstr.replace('@', '')
	if (exchange):
		retstr = retstr + "%" + str(exchange)
	retstr = retstr + "@" + config.jid
	if (userid):
		retstr = retstr + "/" + userid
	return retstr

# This function translates a groupchat JID to an AIM chat room
def jid2aimGroup(jid):
	exchange = 4
	groupid = unicode(jid[1:jid.find('@')].replace('_',' '))
	if (groupid.find('%') != -1):
		exchange = int(groupid[groupid.find('%')+1:])
		groupid = groupid[:groupid.find('%')]
	if (jid.find('/') != -1):
		userid = unicode(jid[jid.find('/'):])
	else:
		userid = None
	return (groupid,userid,exchange)

# This function is called to handle legacy id translation to a JID
translateAccount = aim2jid

# This class handles groupchats with the legacy protocol
class LegacyGroupchat(groupchat.BaseGroupchat):
	def __init__(self, session, resource, ID=None, existing=False, switchboardSession=None):
		groupchat.BaseGroupchat.__init__(self, session, resource, ID)
		self.mungeroom = 0
		if (ID.find('%') == -1):
			# To be used later
			self.mungeroom = 1
		groupid,userid,exchange = jid2aimGroup(self.roomJID())
		debug.log("LegacyGroupchat: \"%s\" \"%s\" created" % (self.roomJID(), groupid))
		self.session.legacycon.createChat(groupid,exchange)

	def removeMe(self):
		groupid,userid,exchange = jid2aimGroup(self.roomJID())
		debug.log("LegacyGroupchat: remove \"%s\" \"%s\"" % (self.roomJID(), groupid))
		self.session.legacycon.leaveChat(groupid)
		groupchat.BaseGroupchat.removeMe(self)
		utils.mutilateMe(self)

	def sendLegacyMessage(self, message):
		groupid,userid,exchange = jid2aimGroup(self.roomJID())
		debug.log("LegacyGroupchat: send message \"%s\" \"%s\" \"%s\"" % (self.roomJID(), groupid, message))
		self.session.legacycon.sendChat(groupid, message)
	
	def sendContactInvite(self, contactJID):
		groupid,userid,exchange = jid2aimGroup(self.roomJID())
		contactid = jid2aim(contactJID)
		debug.log("LegacyGroupchat: send invite \"%s\" \"%s\" \"%s\" \"%s\"" % (self.roomJID(), contactJID, groupid, contactid))
		self.session.legacycon.sendInvite(groupid, contactid)

# This class handles most interaction with the legacy protocol
class LegacyConnection(aimt.AIMConnection):
	""" A glue class that connects to the legacy network """
	def __init__(self, username, password, session):
		debug.log("LegacyConnection: __init__")
		self.session = session
		self.savedShow = None
		self.savedFriendly = None
		aimt.AIMConnection.__init__(self, username, password)
	
	def removeMe(self):
		debug.log("LegacyConnection: removeMe")
		aimt.AIMConnection.removeMe(self)
		utils.mutilateMe(self)

	def jidRes(self, resource):
		to = self.session.jabberID
		if(resource):
			to += "/" + resource
		return to

	def highestResource(self):
		""" Returns highest priority resource """
		return self.session.highestResource()

	def sendMessage(self, dest, resource, body):
		debug.log("LegacyConnection: sendMessage %s %s %s" % (dest, resource, body))
		aimt.AIMConnection.sendMessage(self, dest, body)

	def newResourceOnline(self, resource):
		debug.log("LegacyConnection: newResourceOnline %s" % (resource))
		aimt.AIMConnection.resendBuddies(self, resource)
	
 	def setStatus(self, show, friendly):
		debug.log("LegacyConnection: setStatus %s %s" % (show, friendly))

                if (show=="away" and  not friendly ):
                        friendly="Away"
                elif (show=="dnd" and  not friendly ):
                        friendly="Do Not Disturb"
                elif (show=="xa" and  not friendly ):
                        friendly="Extended Away"
                elif (show=="chat" and  not friendly ):
                        friendly="Free to Chat"

		self.savedShow = show
		self.savedFriendly = friendly

		if (not self.session.ready):
			return

		if (show in ["online", None, "chat"]):
			aimt.AIMConnection.setBack(self, friendly)
			self.session.sendPresence(to=self.session.jabberID, fro=config.jid, show=None)
		else:
			aimt.AIMConnection.setAway(self, friendly)
			self.session.sendPresence(to=self.session.jabberID, fro=config.jid, show=show, status=friendly)

        def buildFriendly(self, status):
		friendly = self.jabberID[:self.jabberID.find('@')]
		if(status and len(status) > 0):
			friendly += " - "
			friendly += status
		if(len(friendly) > 127):
			friendly = friendly[:124] + "..."
		debug.log("Session: buildFriendly(%s) returning \"%s\"" % (self.jabberID, friendly))
		return friendly
	
	def jabberSubscriptionReceived(self, source, subtype):
		debug.log("LegacyConnection: jabberSubscriptionReceived %s %s" % (source, subtype))
		aimt.AIMConnection.jabberSubscriptionReceived(self, source, subtype)

	def userTypingNotification(self, dest, resource, composing):
		debug.log("LegacyConnection: userTypingNotification %s %s" % (dest,composing))
		if (composing):
			aimt.AIMConnection.sendTypingNotify(self, "begin", dest)
		else:
			aimt.AIMConnection.sendTypingNotify(self, "finish", dest)

	def jabberVCardRequest(self, vcard, user):
		debug.log("LegacyConnection: jabberVCardRequest %s" % (user))
		return aimt.AIMConnection.getvCard(self, vcard, user)

	def createChat(self, chatroom, exchange):
		debug.log("LegacyConnection: createChat %s %d" % (chatroom, exchange))
		aimt.AIMConnection.createChat(self, chatroom, exchange)

	def leaveChat(self, chatroom):
		debug.log("LegacyConnection: leaveChat %s" % (chatroom))
		aimt.AIMConnection.leaveChat(self, chatroom)

	def sendChat(self, chatroom, message):
		debug.log("LegacyConnection: sendChat %s %s" % (chatroom, message))
		aimt.AIMConnection.sendChat(self, chatroom, message)

	def sendInvite(self, chatroom, contact):
		debug.log("LegacyConnection: sendInvite %s %s" % (chatroom, contact))
		aimt.AIMConnection.sendInvite(self, chatroom, contact)

	def resourceOffline(self, resource):
		debug.log("LegacyConnection: resourceOffline %s" % (resource))
		pass
